// auxmj.cc
//
// Mark Johnson, 11th October 2001
//
// This is a lightly reworked version of code that Eugene gave
// me for relabelling verbs with the various AUX POS labels
// that his parser uses.
// 
// I use it to transform the original corpus into the format
// that his parser was trained on.

using namespace std;

#include <cctype>
#include <ext/hash_map>
#include <string>

#include "auxmj.h"
#include "sym.h"
#include "symset.h"


inline static void toUpper(string& s)
{
  for (string::iterator i = s.begin(); i != s.end(); ++i)
    *i = toupper(*i);
}


inline static bool
hasAuxSuf(const string& word)
{
  static symset suffixes("'VE 'M 'LL 'D 'S 'RE");
  size_t pos = word.find_first_of("\'");
  if(pos == string::npos) 
    return false;
  string apostrophe = word.substr(pos, word.length()-pos);
  return suffixes.contains(apostrophe);
}

inline static bool
isAux(symbol tag)
{
  static symset auxs("MAHT SHULD WILL WAS OUGHTA AHM NEED MAYE WILLYA WHADDYA HATH HAVE WERE IS HAS MUST DID HAD DO MIGHT WOULD "
		     "SHALL SHOULD OUGHT COULD DOES HAFTA BE KIN CAN ART BEEN DONE ARE DOO MAY AM");
  return auxs.contains(tag);
}

inline static bool
isAuxg(symbol tag)
{
  static symset auxgs("BEIN HAVING BEING");
  return auxgs.contains(tag);
}

inline static bool isVerb(symbol tag) {
  static symset verb_pos("VB VBD VBG VBN VBP VBZ");
  return verb_pos.contains(tag);
}


static symbol auxify(symbol word, symbol tag)
{
  if (isVerb(tag)) {
    static symbol AUX("AUX"), AUXG("AUXG");
    typedef ext::hash_map<symbol,symbol> S_S;
    static S_S word_tag;          // cache previous results
  
    pair<S_S::iterator,bool> it_ins = word_tag.insert(make_pair(word,tag));
    if (it_ins.second == false) {  // is there a cache entry?
      tag = it_ins.first->second;  //  yes. use tag from cache
    }
    else {                         //  no cache entry.
      string w = word.string_reference();
      toUpper(w);
      symbol upperword(w);
      if (isAux(upperword) || hasAuxSuf(w))
	it_ins.first->second = tag = AUX;
      else if (isAuxg(upperword))
	it_ins.first->second = tag = AUXG;
    }
  }
  return tag;
}

void auxify(tree* t)
{
  if (t != NULL) {
    if (t->is_preterminal()) 
      t->label.cat = auxify(t->child->label.cat, t->label.cat);
    auxify(t->child);
    auxify(t->next);
  }
}
