// cbinom_approx.h
//
// Mark Johnson, 6th Nov 2001
//
// Approximation to binomial confidence interval calculation
//  as suggested in Agresti 1996, p 12.
//
// A 95% confidence interval corresponds to z = 1.96
// A 75% confidence interval corresponds to z = 0.69

#include <math.h>

template <typename n_type, typename k_type>
inline double cbinom_lower_p(n_type n, k_type k, double z = 0.69)
{
  double f = double(k)/double(n);
  double z2 = z*z;
  double lower_p = (f+z2/(2*n)-z*sqrt(f/n-f*f/n+z2/(4*n*n)))/(1+z2/n);
  assert(finite(lower_p));
  assert(lower_p < f);
  return lower_p;
}

template <typename n_type, typename k_type>
inline double cbinom_upper_p(n_type n, k_type k, double z = 0.69)
{
  double f = double(k)/double(n);
  double z2 = z*z;
  double upper_p = (f+z2/(2*n)+z*sqrt(f/n-f*f/n+z2/(4*n*n)))/(1+z2/n);
  assert(finite(upper_p));
  assert(upper_p > f);
  return upper_p;
}

