// empty.cc
//
// Mark Johnson, 24th August 2001

#include <algorithm>
#include <cassert>
#include <iterator>
#include <map>
#include <memory>
#include <utility>

#include "empty.h"
#include "sym.h"
#include "utility.h"



std::ostream& operator<< (std::ostream& os, const tree_slash_label& label)
{
  typedef tree_label* tree_label_ptr;
  os << *tree_label_ptr(&label);  // print out standard label
  if (!label.slashes.empty()) {   // do we need to print slashes?
    os << '/';
    if (label.slashes.size()==1)  // how many are there?
      os << *label.slashes.begin();
    else
      os << label.slashes;
  }
  return os;
}
	
typedef tree_node<sI> itree;  // an itree's nodes are labelled with sets of indices

// tree_insides returns an itree whose nodes are labelled with the union of
//  the indices appearing on the corresponding nodes of the tree t.
//
static itree* tree_insides(const tree* t)
{
  assert(t != NULL);
  itree* it = new itree;
  if (t->next) 
    it->next = tree_insides(t->next);
  if (t->child) {
    it->child = tree_insides(t->child);
    for (itree* itc = it->child; itc; itc = itc->next)
      it->label.insert(itc->label.begin(), itc->label.end());
  }
  if (t->label.index) 
    it->label.insert(t->label.index);
  return it;
}

// tree_stree returns an stree whose nodes are labelled with the slash 
//  indices of the corresponding nodes in t.  
//
//  ins should be the isomorphic itree of inside indices (constructed by tree_insides).  
//
//  pre_out and post_out are sets of (outside) indices of the nodes to the left 
//  and to the right of this node respectively
//
static stree* tree_stree(const tree* t, itree* ins, const sI& pre_out, sI& post_out)
{
  assert(t != NULL);
  assert(ins != NULL);

  sI out(pre_out);
  stree* slash = new stree(t->label);
  
  if (t->next) {
    sI next_pre_out(pre_out);
    next_pre_out.insert(ins->label.begin(), ins->label.end());
    slash->next = tree_stree(t->next, ins->next, next_pre_out, post_out);
  }
  
  out.insert(post_out.begin(), post_out.end());

  set_intersection(out.begin(), out.end(),
		   ins->label.begin(), ins->label.end(),
		   inserter(slash->label.slashes, slash->label.slashes.begin()));

  if (t->label.index)
    post_out.insert(t->label.index);

  if (t->child) {
    assert(ins->child);
    slash->child = tree_stree(t->child, ins->child, pre_out, post_out);
  }
  return slash;
}

// tree_stree returns an itree whose nodes are labelled with the
//  set of slash indices of t
//
stree* tree_stree(const tree* t)
{
  std::auto_ptr<itree> insides(tree_insides(t));
  sI pre_out, post_out;
  stree* slash = tree_stree(t, insides.get(), pre_out, post_out);
  return slash;
}

void display_slash_paths(std::ostream& os, const tree* t)
{
  std::auto_ptr<stree> slash(tree_stree(t));
  os << slash.get();
}


