/* read-tree.l
 *
 * (c) Mark Johnson, 23rd August 2001
 */

%option noyywrap
%option fast
%option 8bit
%option prefix="read_tree"

%{
#include "sym.h"
#include "tree.h"

#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <stack>

int readtree_lineno = 1;
const char* readtree_filename = NULL;

inline void message(char* s1, char* s2) {
  fprintf(stderr, "%s:%d: %s %s\n", readtree_filename, readtree_lineno, s1, s2);
}

#define YY_DECL static tree* read_tree()

%}

%s RT
%s RTC
%s FC
%s NC
%s CAT
%x PC
%s IND

%%

%{
  tree* root;           /* tree's root node */
  std::stack<tree*> s;  /* stack of tree node ptrs */
%}

<RT>"("                 assert(s.empty()); s.push(root = new tree); s.top()->label.cat = symbol::root; BEGIN(FC); 

<RTC>"("		assert(s.empty()); s.push(root = new tree); BEGIN(CAT);

<FC>"("			assert(!s.empty()); s.push(s.top()->child = new tree); BEGIN(CAT); 
<FC>[^ \t\n()]+		assert(!s.empty()); s.push(s.top()->child = new tree); s.top()->label.cat = symbol(yytext); BEGIN(NC);
<FC>")"			assert(!s.empty()); s.pop(); if (s.size() == 1) return root; BEGIN(NC);

<NC>")"			assert(!s.empty()); s.pop(); if (s.size() == 1) return root;  
<NC>"("			assert(!s.empty()); s.top() = s.top()->next = new tree; BEGIN(CAT); 

<CAT>"-NONE-"           { assert(!s.empty());  
                          s.top()->label.cat = symbol(yytext);
			  s.push(s.top()->child = new tree);
                         }
<CAT>[A-Z0-9$?*]+	assert(!s.empty()); s.top()->label.cat = symbol(yytext); BEGIN(PC); 
<CAT>[^A-Z0-9 \n\t()$*][^ \n\t()]*  assert(!s.empty()); s.top()->label.cat = symbol(yytext); BEGIN(PC);

<PC>"-"[0-9]+		{ assert(!s.empty()); 
                          assert(s.top()->label.index == 0); 
                          s.top()->label.index = strtoul(yytext+1, NULL, 10); 
			  assert(s.top()->label.index != 0); 
                         }
<PC>"-"[A-Z]+		/* ignore GF tag */
<PC>([=|+-])([^ \t\n()-])+	/* ignore other tags */
<PC>.			unput(yytext[0]); BEGIN(FC);

[ \t]+			/* ignore spaces */
"\n"			++readtree_lineno;   /* increment line count */
.			message("Unexpected character", yytext); std::cerr << "Parse tree so far: " << root << '\n'; abort();

%%

tree* readtree_root(FILE* fp)
{
  yyin = fp;
  BEGIN(RT);
  return read_tree();
}

tree* readtree(FILE* fp)
{
  yyin = fp;
  BEGIN(RTC);
  return read_tree();
}
