// restorer.cc
//
// Mark Johnson, 17th July 2002
// modified Mark Johnson, 19th July 2002 to use verb transitivity
//
// Restores empty nodes and their antecedents in Charniak-style parse trees
//
// Usage:  restorer restorer.dat < parser-trees > parser-trees-with-empty-nodes
//
// Note: You may want to give an absolute path name to restorer.dat as the
// first argument so the restorer program can be run from any directory.

#include <cstdlib>
#include <cstdio>
#include <fstream>
#include <iostream>

#include "restore-empties.h"
#include "sym.h"
#include "transitive-verbs.h"
#include "utility.h"

int main(int argc, char** argv) 
{
  if (argc != 2) {
    std::cerr << "Usage: " << argv[0] << " restorer.dat < parses.txt > emptynodeparses.txt"
	      << std::endl;
    exit(EXIT_FAILURE);
  }

  std::ifstream pattern_stream(argv[1]);
  if (!pattern_stream) {
    std::cerr << "Error in restorer.cc: Could not open restorer data file " << argv[1]
	      << std::endl;
    exit(EXIT_FAILURE);
  }

  TransitiveVerbs tv;
  if (!(pattern_stream >> tv)) {
    std::cerr << "Error in restorer.cc: Failed to read transitive verbs from restorer data file "
	      << argv[1] << std::endl;
    exit(EXIT_FAILURE);
  }

  read_pattern_tree_filename = argv[1];
  FILE* fp = fopen(read_pattern_tree_filename, "r");
  if (fp == NULL) {
    std::cerr << "Error in restorer.cc: Failed to open restorer data file " << argv[1] << std::endl;
    exit(EXIT_FAILURE);
  }

  restore_empties<stree_lex_pattern<select_none, false, false> > restorer;
  restorer.read(fp);

  tree* t;
  while ((t = readtree(stdin)) != NULL) {
    tv.label_transitive_verbs(t);  // add _t suffix to transitive verb POS
    restorer(t);                   // restore empty nodes in t
    tv.remove_transitive_verb_labels(t);
    std::cout << t << std::endl;
    delete t;
  }
}
