// tree.cc
//
// Mark Johnson, 15th June 2001

#include "tree.h"

#include <cctype>
#include <cstdlib>
#include <iostream>


std::ostream& operator<< (std::ostream& os, const tree_label& label)
{
  os << label.cat.c_str();
  if (label.index != 0)
    os << '-' << label.index;
  return os;
}


void read_filenamefile_trees(const char* filenamefile, std::vector<tree*>& trees, 
			     tree* (*reader)(FILE*))
{
  std::ifstream is(filenamefile);
  if (!is) {
    std::cerr << "Error in tree.cc read_filenamefile_trees(): can't open filenamefile " 
	      << filenamefile << std::endl;
    abort();
  }

  std::string filename;
  while (is >> filename) {
    FILE* fp = fopen(filename.c_str(), "r");
    if (fp == NULL) {
      std::cerr << "Error in tree.cc read_filenamefile_trees(): can't open filename " 
		<< filename << std::endl;
      abort();
    }

    readtree_filename = filename.c_str();
    readtree_lineno = 1;

    tree* tp;
    while ((tp = reader(fp))) {
      trees.push_back(tp);
      if (!tp->fillers_include_gaps()) {
	std::cerr << "Error in tree.cc read_filenamefile_trees() in tree datafile "
		  << filename << ":" << readtree_lineno 
		  << ": Filler indices don't include gap indices" << std::endl;
      }
    }

    fclose(fp);
  }
}


void tree_terms(const tree* t, std::vector<symbol>& terms) 
{
  for ( ; t; t=t->next)
    if (t->child)
      tree_terms(t->child, terms);
    else 
      terms.push_back(t->label.cat);
}


